==> rubocop_0.49.1+dfsg.orig.tar.gz: rubocop-0.49.1/lib/rubocop/cop/style/raise_args.rb <==
# frozen_string_literal: true

module RuboCop
  module Cop
    module Style
      # This cop checks the args passed to `fail` and `raise`. For exploded
      # style (default), it recommends passing the exception class and message
      # to `raise`, rather than construct an instance of the error. It will
      # still allow passing just a message, or the construction of an error
      # with more than one argument.
      #
      # The exploded style works identically, but with the addition that it
      # will also suggest constructing error objects when the exception is
      # passed multiple arguments.
      #
      # @example
      #
      #   # EnforcedStyle: exploded
      #
      #   # bad
      #   raise StandardError.new("message")
      #
      #   # good
      #   raise StandardError, "message"
      #   fail "message"
      #   raise MyCustomError.new(arg1, arg2, arg3)
      #   raise MyKwArgError.new(key1: val1, key2: val2)
      #
      # @example
      #
      #   # EnforcedStyle: compact
      #
      #   # bad
      #   raise StandardError, "message"
      #   raise RuntimeError, arg1, arg2, arg3
      #
      #   # good
      #   raise StandardError.new("message")
      #   raise MyCustomError.new(arg1, arg2, arg3)
      #   fail "message"
      class RaiseArgs < Cop
        include ConfigurableEnforcedStyle

        EXPLODED_MSG = 'Provide an exception class and message ' \
          'as arguments to `%s`.'.freeze
        COMPACT_MSG = 'Provide an exception object ' \
          'as an argument to `%s`.'.freeze

        def on_send(node)
          return unless node.command?(:raise) || node.command?(:fail)

          case style
          when :compact
            check_compact(node)
          when :exploded
            check_exploded(node)
          end
        end

        private

        def autocorrect(node)
          new_exception = if style == :compact
                            correction_exploded_to_compact(node.arguments)
                          else
                            correction_compact_to_exploded(node.first_argument)
                          end
          replacement = "#{node.method_name} #{new_exception}"

          ->(corrector) { corrector.replace(node.source_range, replacement) }
        end

        def correction_compact_to_exploded(node)
          exception_node, _new, message_node = *node

          message = message_node && message_node.source

          correction = exception_node.const_name.to_s
          correction = "#{correction}, #{message}" if message

          correction
        end

        def correction_exploded_to_compact(node)
          exception_node, *message_nodes = *node

          messages = message_nodes.map(&:source).join(', ')

          "#{exception_node.const_name}.new(#{messages})"
        end

        def check_compact(node)
          if node.arguments.size > 1
            add_offense(node, :expression, message(node.method_name)) do
              opposite_style_detected
            end
          else
            correct_style_detected
          end
        end

        def check_exploded(node)
          return correct_style_detected unless node.arguments.one?

          first_arg = node.first_argument

          return unless first_arg.send_type? && first_arg.method?(:new)

          return if acceptable_exploded_args?(first_arg.arguments)

          add_offense(node, :expression, message(node.method_name)) do
            opposite_style_detected
          end
        end

        def acceptable_exploded_args?(args)
          # Allow code like `raise Ex.new(arg1, arg2)`.
          return true if args.size > 1

          # Disallow zero arguments.
          return false if args.empty?

          arg = args.first

          # Allow code like `raise Ex.new(kw: arg)`.
          # Allow code like `raise Ex.new(*args)`.
          arg.hash_type? || arg.splat_type?
        end

        def message(method)
          if style == :compact
            format(COMPACT_MSG, method)
          else
            format(EXPLODED_MSG, method)
          end
        end
      end
    end
  end
end

==> rubocop_0.52.1+dfsg.orig.tar.gz: rubocop-0.52.1/lib/rubocop/cop/style/raise_args.rb <==
# frozen_string_literal: true

module RuboCop
  module Cop
    module Style
      # This cop checks the args passed to `fail` and `raise`. For exploded
      # style (default), it recommends passing the exception class and message
      # to `raise`, rather than construct an instance of the error. It will
      # still allow passing just a message, or the construction of an error
      # with more than one argument.
      #
      # The exploded style works identically, but with the addition that it
      # will also suggest constructing error objects when the exception is
      # passed multiple arguments.
      #
      # @example EnforcedStyle: exploded (default)
      #   # bad
      #   raise StandardError.new("message")
      #
      #   # good
      #   raise StandardError, "message"
      #   fail "message"
      #   raise MyCustomError.new(arg1, arg2, arg3)
      #   raise MyKwArgError.new(key1: val1, key2: val2)
      #
      # @example EnforcedStyle: compact
      #   # bad
      #   raise StandardError, "message"
      #   raise RuntimeError, arg1, arg2, arg3
      #
      #   # good
      #   raise StandardError.new("message")
      #   raise MyCustomError.new(arg1, arg2, arg3)
      #   fail "message"
      class RaiseArgs < Cop
        include ConfigurableEnforcedStyle

        EXPLODED_MSG = 'Provide an exception class and message ' \
          'as arguments to `%<method>s`.'.freeze
        COMPACT_MSG = 'Provide an exception object ' \
          'as an argument to `%<method>s`.'.freeze

        def on_send(node)
          return unless node.command?(:raise) || node.command?(:fail)

          case style
          when :compact
            check_compact(node)
          when :exploded
            check_exploded(node)
          end
        end

        def autocorrect(node)
          replacement = if style == :compact
                          correction_exploded_to_compact(node)
                        else
                          correction_compact_to_exploded(node)
                        end

          ->(corrector) { corrector.replace(node.source_range, replacement) }
        end

        private

        def correction_compact_to_exploded(node)
          exception_node, _new, message_node = *node.first_argument

          message = message_node && message_node.source

          correction = exception_node.const_name.to_s
          correction = "#{correction}, #{message}" if message

          "#{node.method_name} #{correction}"
        end

        def correction_exploded_to_compact(node)
          exception_node, *message_nodes = *node.arguments
          return node.source if message_nodes.size > 1

          argument = message_nodes.first.source
          "#{node.method_name} #{exception_node.const_name}.new(#{argument})"
        end

        def check_compact(node)
          if node.arguments.size > 1
            add_offense(node) do
              opposite_style_detected
            end
          else
            correct_style_detected
          end
        end

        def check_exploded(node)
          return correct_style_detected unless node.arguments.one?

          first_arg = node.first_argument

          return unless first_arg.send_type? && first_arg.method?(:new)

          return if acceptable_exploded_args?(first_arg.arguments)

          add_offense(node) do
            opposite_style_detected
          end
        end

        def acceptable_exploded_args?(args)
          # Allow code like `raise Ex.new(arg1, arg2)`.
          return true if args.size > 1

          # Disallow zero arguments.
          return false if args.empty?

          arg = args.first

          # Allow code like `raise Ex.new(kw: arg)`.
          # Allow code like `raise Ex.new(*args)`.
          arg.hash_type? || arg.splat_type?
        end

        def message(node)
          if style == :compact
            format(COMPACT_MSG, method: node.method_name)
          else
            format(EXPLODED_MSG, method: node.method_name)
          end
        end
      end
    end
  end
end
